@@ -36,7 +36,7 @@ namespace polymake { namespace polytope {
   This amounts to summing all cocircuit equations corresponding to the orbit of each ridge.
  */
 template<typename Scalar, typename SetType>
-ListMatrix<SparseVector<int> > 
+ListMatrix<SparseVector<int>>
 symmetrized_cocircuit_equations_0_impl(int d,
                                        const Matrix<Scalar>& V,
                                        const IncidenceMatrix<>& VIF,
@@ -44,7 +44,7 @@ symmetrized_cocircuit_equations_0_impl(int d,
                                        const Array<SetType>& interior_ridge_reps,
                                        const Array<SetType>& facet_reps,
                                        perl::OptionSet options,
-                                       bool partial_equations) 
+                                       bool partial_equations)
 {
    const std::string filename = options["filename"];
    std::ofstream outfile(filename.c_str(), std::ios_base::trunc);
@@ -58,7 +58,7 @@ symmetrized_cocircuit_equations_0_impl(int d,
    for (const auto& rep : facet_reps)
       index_of_facet[rep] = ++ct;
 
-   const int 
+   const int
       n_facets(index_of_facet.size()),
       n(V.rows());
    ListMatrix<SparseVector<int>> cocircuit_eqs(0, n_facets);
@@ -107,7 +107,7 @@ symmetrized_foldable_cocircuit_equations_0_impl(int d,
                                                 const Array<SetType>& interior_ridge_reps,
                                                 const Array<SetType>& facet_reps,
                                                 perl::OptionSet options,
-                                                bool partial_equations) 
+                                                bool partial_equations)
 {
    const group::PermlibGroup sym_group(generators);
 
@@ -120,12 +120,8 @@ symmetrized_foldable_cocircuit_equations_0_impl(int d,
    int n_facet_reps(0);
    for (const auto& rep : facet_reps)
       index_of_facet[rep] = n_facet_reps++;
-   
-   ListMatrix<SparseVector<int>> cocircuit_eqs(0, 2*n_facet_reps);
 
-   // use int instead of Rational to save time;
-   //   we don't use row reductions (experimentally bad)
-   SparseVector<int> eq_0_first, eq_1_first;
+   ListMatrix<SparseVector<int>> cocircuit_eqs(0, 2*n_facet_reps);
 
    // for each interior ridge rho and c in {0,1}:
    //   sum_{sigma > rho, orientation=+} x_{c,sigma} = sum_{sigma > rho, orientation=-} x_{1-c,sigma}
@@ -137,11 +133,14 @@ symmetrized_foldable_cocircuit_equations_0_impl(int d,
          time(&current_time);
          cerr << ct << " " << difftime(current_time, start_time) << endl;
       }
-      eq_0_first = SparseVector<int>(2*n_facet_reps);
-      eq_1_first = SparseVector<int>(2*n_facet_reps); 
+
+      // use int instead of Rational to save time;
+      //   we don't use row reductions (experimentally bad)
+      SparseVector<int> eq_0_first(2*n_facet_reps);
+      SparseVector<int> eq_1_first(2*n_facet_reps);
       const SparseVector<Scalar> nv = null_space(V.minor(rho, All)).row(0);
-      int row_index(0); 
-      for (typename Entire<Rows<Matrix<Scalar>>>::const_iterator vit = entire(rows(V)); !vit.at_end(); ++vit, ++row_index) {
+      int row_index = 0;
+      for (auto vit = entire(rows(V)); !vit.at_end(); ++vit, ++row_index) {
          const int orientation = sign(nv * (*vit));
          if (orientation != 0) {
             SetType facet(rho);
@@ -149,15 +148,15 @@ symmetrized_foldable_cocircuit_equations_0_impl(int d,
             facet += row_index;
             const SetType this_facet(sym_group.lex_min_representative(facet));
             if (partial_equations && !index_of_facet.exists(this_facet)) continue;
-            const int 
+            const int
                iof (index_of_facet[this_facet]),
                mult(sym_group.setwise_stabilizer(this_facet).order());
             if (orientation>0) {
-               eq_0_first[2*iof] +=  mult;
-               eq_1_first[2*iof+1] +=  mult;
+               eq_0_first[2*iof]   += mult;
+               eq_1_first[2*iof+1] += mult;
             } else {
-               eq_0_first[2*iof+1] +=  -mult;
-               eq_1_first[2*iof] +=  -mult;
+               eq_0_first[2*iof+1] -= mult;
+               eq_1_first[2*iof]   -= mult;
             }
          }
       }
@@ -172,7 +171,7 @@ symmetrized_foldable_cocircuit_equations_0_impl(int d,
             wrap(outfile) << eq_0_first << "\n" << eq_1_first << endl;
       }
    }
-   return cocircuit_eqs;     
+   return cocircuit_eqs;
 }
 
 } }
@@ -183,4 +182,3 @@ symmetrized_foldable_cocircuit_equations_0_impl(int d,
 // indent-tabs-mode:nil
 // End:
 
- 
