@@ -51,7 +51,7 @@ UserFunction4perl("# @category Producing a polytope from polytopes"
                   "# @param Polytope P"
                   "# @return Polytope"
                   "# @example [prefer cdd]" 
-                  "# > $p = new Polytope(VERTICES=>[[1,1.3,0.5],[1,0.2,1.2],[1,0.1,-1.5],[1,-1.4,0.2]]);"
+                  "# > $p = new Polytope(VERTICES=>[[1,13/10,1/2],[1,1/5,6/5],[1,1/10,-3/2],[1,-7/5,1/5]]);"
                   "# > $ih = integer_hull($p);"
                   "# > print $ih->VERTICES;"
                   "# | 1 -1 0"
