@@ -1,4 +1,4 @@
-#  Copyright (c) 1997-2017
+#  Copyright (c) 1997-2018
 #  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
 #  http://www.polymake.org
 #
@@ -15,6 +15,7 @@
 
 use strict;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 #################################################################################
 #
@@ -70,7 +71,7 @@ my $help_line_start=qr{^[ \t]*\#[ \t]*}m;
 
 sub add {
    my ($self, $path, $text, $signature)=@_;
-   unless (is_ARRAY($path)) {
+   unless (is_array($path)) {
       $path=[ split m'/', $path ];
    }
    my ($source_file, $source_line, @example_start_lines);
@@ -85,7 +86,7 @@ sub add {
          # in embedded rules, compiler sees everything on a single (last) line because of a macro
          my $line = $source_file =~ /\.(?:cc|cpp|C|h|hh|H)$/ ? $source_line-@lines : $source_line;
          foreach (@lines) {
-            if (/$help_line_start \@example\s /xo) {
+            if (/$help_line_start \@example (?:\s|$)/xo) {
                push @example_start_lines, $line;
             }
             ++$line;
@@ -122,7 +123,7 @@ sub add {
          }
 
          my ($opt_group, @option_lists);
-         while ($text =~ s/$help_line_start \@($id_re) [ \t]+ ( .*\n (?:(?>$help_line_start) (?! \@$id_re) .*\n)* )//xom) {
+         while ($text =~ s/$help_line_start \@($id_re) (?:[ \t]+|$) ( .*\n (?:(?>$help_line_start) (?! \@$id_re) .*\n)* )//xom) {
             my ($tag, $value)=(lc($1), $2);
             sanitize_help($value);
 
@@ -338,7 +339,8 @@ sub add {
 }
 #################################################################################
 sub add_tparams {
-   my $self=shift;
+   my $self = shift;
+   $self->annex->{mandatory_tparams} = shift;
    $self->annex->{tparam} //= [ map { [ $_ ] } @_ ];
 }
 #################################################################################
@@ -397,7 +399,7 @@ sub write_function_text {
                my $keys;
                $writer->function_options($opt_group->text, map { defined($keys=$_->annex->{keys}) ? @$keys : () } $opt_group, @{$opt_group->related});
             } else {
-               my $comment=local_shift($opt_group);
+               my $comment = local shift @$opt_group;
                $writer->function_options($comment, @$opt_group);
             }
          }
@@ -419,7 +421,7 @@ sub write_function_text {
                my $keys;
                push @options, map { defined($keys=$_->annex->{keys}) ? @$keys : () } $opt_group, @{$opt_group->related};
             } else {
-               local_shift($opt_group);
+               local shift @$opt_group;
                push @options, @$opt_group;
             }
          }
@@ -461,7 +463,7 @@ sub write_text {
       $writer->description($self->text);
       if (exists $self->annex->{keys}) {
          my $keys;
-         $writer->topic_keys(map { defined($keys=$_->annex->{keys}) ? @$keys : () } $self, @{$self->related});
+         $writer->topics_keys(map { defined($keys=$_->annex->{keys}) ? @$keys : () } $self, @{$self->related});
       }
 
       if ($full) {
@@ -585,7 +587,7 @@ sub argument_completions {
                   push @matching, grep { /^\Q$prefix\E/ } map { $_->[1] } @$keys;
                }
             } else {
-               local_shift($opt_group);
+               local shift @$opt_group;
                push @matching, grep { /^\Q$prefix\E/ } map { $_->[1] } @$opt_group;
             }
          }
@@ -687,7 +689,7 @@ redefine Help;
 require Polymake::Core::HelpAsPlainText;
 
 #################################################################################
-package _::Example;
+package Polymake::Core::InteractiveHelp::Example;
 
 use Polymake::Struct (
    [ new => '$$$@' ],
@@ -698,7 +700,7 @@ use Polymake::Struct (
 );
 
 #################################################################################
-package __::Specialization;
+package Polymake::Core::InteractiveHelp::Specialization;
 
 use Polymake::Struct (
    [ new => '$$' ],
